@@ -22,6 +22,7 @@
 # 3. This notice may not be removed or altered from any source distribution.
 
 import glob, os, sys
+import cross_bdist_wininst
 
 from ez_setup import use_setuptools
 from distutils.command.build import build
@@ -29,43 +30,29 @@ from distutils.command.build_ext import build_ext
 use_setuptools()
 
 import setuptools
-import setup
+import urllib
+import zipfile
 
-class DocBuilder(setuptools.Command):
-    description = "Builds the documentation"
-    user_options = []
+from setup import get_setup_args, DocBuilder
 
-    def initialize_options(self):
-        pass
+AMALGAMATION_ROOT = "amalgamation"
 
-    def finalize_options(self):
-        pass
-
-    def run(self):
-        import os, stat
-
-        try:
-            import docutils.core
-            import docutilsupport
-        except ImportError:
-            print "Error: the build-docs command requires docutils and SilverCity to be installed"
-            return
-
-        docfiles = {
-            "usage-guide.html": "usage-guide.txt",
-            "install-source.html": "install-source.txt",
-            "install-source-win32.html": "install-source-win32.txt"
-        }
-
-        os.chdir("doc")
-        for dest, source in docfiles.items():
-            if not os.path.exists(dest) or os.stat(dest)[stat.ST_MTIME] < os.stat(source)[stat.ST_MTIME]:
-                print "Building documentation file %s." % dest
-                docutils.core.publish_cmdline(
-                    writer_name='html',
-                    argv=["--stylesheet=default.css", source, dest])
-
-        os.chdir("..")
+def get_amalgamation():
+    """Download the SQLite amalgamation if it isn't there, already."""
+    if os.path.exists(AMALGAMATION_ROOT):
+        return
+    os.mkdir(AMALGAMATION_ROOT)
+    print "Downloading amalgation."
+    urllib.urlretrieve("http://sqlite.org/sqlite-amalgamation-3_6_1.zip", "tmp.zip")
+    zf = zipfile.ZipFile("tmp.zip")
+    files = ["sqlite3.c", "sqlite3.h"]
+    for fn in files:
+        print "Extracting", fn
+        outf = open(AMALGAMATION_ROOT + os.sep + fn, "wb")
+        outf.write(zf.read(fn))
+        outf.close()
+    zf.close()
+    os.unlink("tmp.zip")
 
 class AmalgamationBuilder(build):
     description = "Build a statically built pysqlite using the amalgamtion."
@@ -78,9 +65,10 @@ class MyBuildExt(build_ext):
     amalgamation = False
 
     def build_extension(self, ext):
+        get_amalgamation()
         if self.amalgamation:
-            ext.sources.append("sqlite3.c")
-        build_ext.build_extension(self, ext)
+            ext.sources.append(os.path.join(AMALGAMATION_ROOT, "sqlite3.c"))
+            build_ext.build_extension(self, ext)
 
     def __setattr__(self, k, v):
         # Make sure we don't link against the SQLite library, no matter what setup.cfg says
@@ -90,11 +78,9 @@ class MyBuildExt(build_ext):
 
 
 def main():
-    setup_args = setup.get_setup_args()
-    setup_args["extras_require"] = {"build_docs": ["docutils", "SilverCity"]}
+    setup_args = get_setup_args()
     setup_args["test_suite"] = "pysqlite2.test.suite"
-    setup_args["cmdclass"] = {"build_docs": DocBuilder, "build_ext": MyBuildExt, "build_static": AmalgamationBuilder}
-    setup_args["extras_require"] = {"build_docs": ["docutils", "SilverCity"]}
+    setup_args["cmdclass"].update({"build_docs": DocBuilder, "build_ext": MyBuildExt, "build_static": AmalgamationBuilder, "cross_bdist_wininst": cross_bdist_wininst.bdist_wininst})
     setuptools.setup(**setup_args)
 
 if __name__ == "__main__":
